﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Castle.ActiveRecord;
using Castle.ActiveRecord.Framework;
using Castle.ActiveRecord.Framework.Config;
using System.Reflection;
using System.Collections;
using Castle.ActiveRecord.Queries;

namespace SpeedTest3
{
    [ActiveRecord( Table="CAKsiazka" )]
    public class Ksiazka : ActiveRecordBase<Ksiazka>
    {
        public Ksiazka() { K_Egzemplarz = new List<Egzemplarz>(); }

        [PrimaryKey]
        public int ID { get; set; }

        [Property]
        public string Tytul { get; set; }

        [Property]
        public string Autor { get; set; }

        [HasMany( typeof( Egzemplarz ), Lazy=true, Fetch=FetchEnum.Unspecified, Table="Egzemplarz", ColumnKey="ID_Ksiazka", Cascade=ManyRelationCascadeEnum.All )]
        public IList<Egzemplarz> K_Egzemplarz { get; set; }
    }
    [ActiveRecord( Table="CAEgzemplarz" )]
    public class Egzemplarz : ActiveRecordBase<Egzemplarz>
    {
        public Egzemplarz() { }

        [PrimaryKey]
        public int ID { get; set; }

        [Property]
        public string Sygnatura { get; set; }

        [Property]
        public int Status { get; set; }
        
        [BelongsTo( "ID_Ksiazka", Fetch=FetchEnum.Join )]
        public Ksiazka Ksiazka { get; set; }
    }
    class Program
    {
        static void Main( string[] args )
        {
            IConfigurationSource source = ActiveRecordSectionHandler.Instance;
            ActiveRecordStarter.Initialize( Assembly.GetAssembly( typeof( Program ) ), source );
            ActiveRecordStarter.DropSchema();
            ActiveRecordStarter.CreateSchema();


            // 1. Tworzenie obiektów
            {
                DateTime start = DateTime.Now;
                int count = 0;
                for ( int i=0; i<5000; ++i )
                {
                    var k = new Ksiazka { Autor = "A"+i.ToString(), Tytul = "T"+i.ToString() };
                    for ( int j=0; j<20; ++j, ++count )
                    {
                        var e = new Egzemplarz { Sygnatura = "S-"+i.ToString()+"-"+j.ToString() };
                        k.K_Egzemplarz.Add( e );
                    }
                    k.Save();
                }
                DateTime stop = DateTime.Now;
                Console.WriteLine( ( stop - start ).TotalMilliseconds.ToString() );
            }

            // 1a. Tworzenie obiektów (UoW)
            //{
            //     Żeby zadziałał SessionScope, trzeby ustawić [PrimaryKey( PrimaryKeyType.Assigned )]
                
            //    //var sf = ActiveRecordMediator.GetSessionFactoryHolder();
            //    //var s = sf.CreateSession( typeof( Ksiazka ) );
            //    //s.FlushMode = NHibernate.FlushMode.Never;
            //    using ( SessionScope scope = new SessionScope( FlushAction.Never ) )
            //    {
            //        DateTime start = DateTime.Now;
            //        int count = 0;
            //        for ( int i=0; i<5000; ++i )
            //        {
            //            var k = new Ksiazka { ID = i, Autor = "A"+i.ToString(), Tytul = "T"+i.ToString() };

            //            for ( int j=0; j<20; ++j, ++count )
            //            {
            //                var e = new Egzemplarz { ID = count, Sygnatura = "S-"+i.ToString()+"-"+j.ToString() };
            //                e.Create();
            //                k.K_Egzemplarz.Add( e );
            //            }
            //            k.Create();

            //        }
            //        scope.Flush();
            //        DateTime stop = DateTime.Now;
            //        Console.WriteLine( ( stop - start ).TotalMilliseconds.ToString() );
            //    }
            //}

            // 2. Wyciąganie danych
            //{
            //    double[] czasy1 = new double[10];
            //    double[] czasy2 = new double[10];
            //    for ( int x = 0; x<10; x++ )
            //    {
            //        DateTime t1 = DateTime.Now;

            //        HqlBasedQuery query = new HqlBasedQuery(
            //            typeof( Ksiazka ),
            //            "from Egzemplarz e left join e.Ksiazka k" );

            //        ArrayList result = (ArrayList)ActiveRecordMediator.ExecuteQuery( query );

            //        int liczba = 0;
            //        foreach ( var c in result )
            //            if ( ( ( c as object[] )[0] as Egzemplarz ).Sygnatura.EndsWith( "5" ) )
            //                ++liczba;

            //        //Console.WriteLine( liczba.ToString() );
            //        DateTime t2 = DateTime.Now;

            //        foreach ( var c in result )
            //        {
            //            var e = ( ( c as object[] )[0] as Egzemplarz );
            //            if ( e.Sygnatura.EndsWith( "5" ) )
            //            {
            //                e.Status++;
            //                e.Save();
            //            }
            //        }

            //        DateTime t3 = DateTime.Now;
            //        czasy1[x] = ( t2 - t1 ).TotalMilliseconds;
            //        czasy2[x] = ( t3 - t2 ).TotalMilliseconds;
            //    }
            //    Console.WriteLine( czasy1.Sum()/10 );
            //    Console.WriteLine( czasy2.Sum()/10 );
            //}
        }
    }
}
